" Kuka Robot Language file type plugin for Vim
" Language: Kuka Robot Language
" Maintainer: Patrick Knosowski <knosowski@graeff.de>
" Version: 0.5.9
" Last Change: 05. Oct 2016
" Credits:
"
" TODO  create path more narrow
"       Find signal variables from cursorposition on $in and $out 
"
" Suggestions of improvement are very welcome. Please email me!
"

" Only do this when not done yet for this buffer
if exists("b:did_ftplugin")
  finish
endif
let b:did_ftplugin = 1

let s:keepcpo= &cpo
set cpo&vim

" init for optional messages
if !exists("*s:KrlVerboseEcho()")
  let g:krlVerboseMsgSet = 0
  function s:KrlVerboseEcho(msg)
    if !exists("g:krlNoVerbose") || g:krlNoVerbose!=1
      if g:krlVerboseMsgSet == 0
        let g:krlVerboseMsgSet = 1
        echo " "
        echo "NOTE: This feature is currently in BETA status!"
        echo "Switch verbose messages off with \":let g:krlNoVerbose=1\" any time. You may put this in your .vimrc"
        echo "You may still get messages from vim even with g:krlNoVerbose set."
        echo " "
      endif
      echo a:msg
    endif
  endfunction
endif

" default on; no option
setlocal commentstring=;%s
setlocal comments=:;
setlocal suffixes+=.dat
setlocal suffixesadd+=.src,.sub,.dat
let b:undo_ftplugin = "setlocal com< cms< su< sua<"

" matchit support
if exists("loaded_matchit")
  let b:match_words = '^\s*\<if\>:^\s*\<else\>:^\s*\<endif\>,'
         \.'^\s*\<\(for\|while\|loop\|repeat\)\>:^\s*\<exit\>:^\s*\<\(end\(for\|while\|loop\)\|until\)\>,'
         \.'^\s*\<switch\>:^\s*\<case\>:^\s*\<default\>:^\s*\<endswitch\>,'
         \.'^\s*\(global\s\+\)\?\<def\(fct\)\?\>:^\s*\<resume\>:^\s*\<return\>:^\s*\<end\(fct\)\?\>,'
         \.'^\s*\<defdat\>:^\s*\<enddat\>'
  let b:match_ignorecase = 1 " KRL does ignore case
endif

" default on; optional
" folding
if has("folding") &&
      \(!exists("g:krlCloseFolds") || g:krlCloseFolds!=2)
  "
  if !exists("*KrlFoldText")
    function! KrlFoldText()
      let l:strLine = getline(v:foldstart)
      let l:sub = substitute(l:strLine, '\v\c(;\s*<FOLD>\s\s?|;[^;]*$)', '', 'g')
      return l:sub
    endfunction
  endif
  "
  if !exists("*s:KrlIsVkrc")
    function s:KrlIsVkrc()
      if bufname("%") =~ '\c\v(folge|up|makro(saw|sps|step|trigger)?)\d*.src'
        for l:s in range(1,8)
          if getline(l:s) =~ '\c\v^\s*\&param\s+tpvw_version\s*.*$'
            return 1
          endif
        endfor
      endif
      return 0
    endfunction
  endif
  "
  " setting starting behavior
  setlocal foldmethod=marker
  setlocal foldtext=KrlFoldText()
  if exists("g:krlCloseFolds") && g:krlCloseFolds==1
        \|| s:KrlIsVkrc()
    " close all folds; too sad, this is case sensitive.
    " For vkrc force closing all folds 
    " work around false fold matching in VKRC files because of nested
    " regular folds within movement folds
    setlocal foldmarker=FOLD,ENDFOLD
  else
    " close only PTP|LIN|CIRC movement folds, also case sensitive
    setlocal foldmarker=%CMOVE,ENDFOLD
  endif
  let b:undo_ftplugin = b:undo_ftplugin." fdm< fdt< fmr<"
  "
  " mappings if g:krlFoldKeyMap is set and folding is not disabled
  if exists("g:krlFoldKeyMap") && g:krlFoldKeyMap==1
    if s:KrlIsVkrc()
      " closing move folds fails in VKRC files
      nnoremap <silent><buffer> <F3> :setlocal foldlevel=0<CR>
    else
    " closing all folds (well, case sensitiv)
    nnoremap <silent><buffer> <F3> :setlocal foldmarker=FOLD,ENDFOLD<CR>
    endif
    if s:KrlIsVkrc()
      " closing move folds fails in VKRC files
      nnoremap <silent><buffer> <F2> :setlocal foldlevel=1<CR>
    else
      " closing only move folds
      nnoremap <silent><buffer> <F2> :setlocal foldmarker=%CMOVE,ENDFOLD<CR>
    endif
  endif
  "
endif " has("folding") || g:krlCloseFolds==2

if !exists("g:krlNoKeyWord") || g:krlNoKeyWord!=1
  " make enums and sysvars a word including # and $
  setlocal iskeyword+=#,$
  let b:undo_ftplugin = b:undo_ftplugin." isk<"
endif

" Program, System, TP, R1 and KRC folders are root for gf, :find etc
if !exists("g:krlNoPath") || g:krlNoPath!=1
  setlocal path=
        \.,
        \./**,
        \./../Program/**,
        \./../System/**,
        \./../Mada/**,
        \./../TP/**,
        \./../R1/**,
        \./../STEU/**,
        \./../KRC/**,
        \./../../Program/**,
        \./../../System/**,
        \./../../Mada/**,
        \./../../TP/**,
        \./../../R1/**,
        \./../../STEU/**,
        \./../../KRC/**,
        \./../../../Program/**,
        \./../../../System/**,
        \./../../../Mada/**,
        \./../../../TP/**,
        \./../../../R1/**,
        \./../../../STEU/**,
        \./../../../KRC/**,
        \./../../../../Program/**,
        \./../../../../System/**,
        \./../../../../Mada/**,
        \./../../../../TP/**,
        \./../../../../R1/**,
        \./../../../../STEU/**,
        \./../../../../KRC/**,
        \./../../../../../R1/**,
        \./../../../../../STEU/**,
        \./../../../../../KRC/**,
        \./../../../../../../KRC/**,
        \,
  let b:undo_ftplugin = b:undo_ftplugin." pa<"
endif

" default off; optional
" mappings for [[ ]] ...
if exists("g:krlMoveAroundKeyMap") && g:krlMoveAroundKeyMap==1
  "
  " if !exists("*KrlNTimesSearch()")
    " function KrlNTimesSearch(sSearchPattern,sFlags)
      " let nCount=v:count1
      " while nCount>0
        " let nCount=nCount-1
        " let command=":call search('".a:sSearchPattern."',\"".a:sFlags."\")"
        " exec command
      " endwhile
    " endfunction
  " endif
  "
  " Move around functions
  " nnoremap <silent><buffer> [[ :call KrlNTimesSearch('\c^\s*\(global\)\?\s*def\(fct\\|dat\)\?\>', "bsW")<CR>
  " vnoremap <silent><buffer> [[ :<C-U>exe "normal! gv"<Bar>call KrlNTimesSearch('\c^\s*\(global\)\?\s*def\(fct\\|dat\)\?\>', "bsW")<CR>
  " nnoremap <silent><buffer> ]] :call KrlNTimesSearch('\c^\s*\(global\)\?\s*def\(fct\\|dat\)\?\>', "sW")<CR>
  " vnoremap <silent><buffer> ]] :<C-U>exe "normal! gv"<Bar>call KrlNTimesSearch('\c^\s*\(global\)\?\s*def\(fct\\|dat\)\?\>', "sW")<CR>
  " nnoremap <silent><buffer> [] :call KrlNTimesSearch('\c^\s*end\(fct\\|dat\)\?\>', "bsWe")<CR>
  " vnoremap <silent><buffer> [] :<C-U>exe "normal! gv"<Bar>call KrlNTimesSearch('\c^\s*end\(fct\\|dat\)\?\>', "bsWe")<CR>
  " nnoremap <silent><buffer> ][ :call KrlNTimesSearch('\c^\s*end\(fct\\|dat\)\?\>', "sWe")<CR>
  " vnoremap <silent><buffer> ][ :<C-U>exe "normal! gv"<Bar>call KrlNTimesSearch('\c^\s*end\(fct\\|dat\)\?\>', "sWe")<CR>
  nnoremap <silent><buffer> [[ ?\c\v^\s*(global)\?\s*def(fct\|dat)\?><CR>
  vnoremap <silent><buffer> [[ ?\c\v^\s*(global)\?\s*def(fct\|dat)\?><CR>
  nnoremap <silent><buffer> ]] /\c\v^\s*(global)?\s*def(fct\|dat)?><CR>
  vnoremap <silent><buffer> ]] /\c\v^\s*(global)?\s*def(fct\|dat)?><CR>
  nnoremap <silent><buffer> [] ?\c\v^\s*end(fct\|dat)\?><CR>
  vnoremap <silent><buffer> [] ?\c\v^\s*end(fct\|dat)\?><CR>
  nnoremap <silent><buffer> ][ /\c\v^\s*end(fct\|dat)?><CR>
  vnoremap <silent><buffer> ][ /\c\v^\s*end(fct\|dat)?><CR>
  "
  " Move around comments
  " nnoremap <silent><buffer> [; :call search('^\(\s*;.*\n\)\@<!\(\s*;\)', "bsW")<CR>
  " vnoremap <silent><buffer> [; :<C-U>exe "normal! gv"<Bar>call search('^\(\s*;.*\n\)\@<!\(\s*;\)', "bsW")<CR>
  " nnoremap <silent><buffer> ]; :call search('\%(^\s*;.*\n\)\%(^\s*;\)\@!', "sWe")<CR>
  " vnoremap <silent><buffer> ]; :<C-U>exe "normal! gv"<Bar>call search('\%(^\s*;.*\n\)\%(^\s*;\)\@!', "sWe")<CR>
  nnoremap <silent><buffer> [; ?^\(\s*;.*\n\)\@<!\(\s*;\)<CR>
  vnoremap <silent><buffer> [; ?^\(\s*;.*\n\)\@<!\(\s*;\)<CR>
  nnoremap <silent><buffer> ]; /\(^\s*;.*\n\)\(^\s*;\)\@!<CR>
  vnoremap <silent><buffer> ]; /\(^\s*;.*\n\)\(^\s*;\)\@!<CR>
  "
endif

" mapping for gd
if exists("g:krlGoDefinitionKeyMap") && g:krlGoDefinitionKeyMap==1
  nnoremap <silent><buffer> gd :call KrlGoDefinition()<CR>
  nnoremap <silent><buffer> <leader>f :cclose\|vimgrep! /\v\c^\s*(global\s+)?def(fct)?>/j % \|cwindow 4\|setf krl<cr>
endif

" auto insert comment char when i_<CR>, o or O on a comment line
if exists("g:krlAutoComment") && g:krlAutoComment==1
  " add comment char if Enter, o or O on a comment line
  setlocal formatoptions+=r
  setlocal formatoptions+=o
  let b:undo_ftplugin = b:undo_ftplugin." fo<"
endif

" format comments
if exists("g:krlFormatComments") && g:krlFormatComments==1
  if &textwidth ==# 0
    " 52 Chars do match on the teach pendant
    setlocal textwidth=52
    let b:undo_ftplugin = b:undo_ftplugin." tw<"
  endif
  setlocal formatoptions-=t
  setlocal formatoptions+=l
  setlocal formatoptions+=j
  if stridx(b:undo_ftplugin, " fo<")==(-1)
    let b:undo_ftplugin = b:undo_ftplugin." fo<"
  endif
  "
  if !exists("*KrlFormatComments")

    function KrlFormatComments()
      "
      execute "normal" "m'"
      execute 0
      let l:numCurrLine = 1
      let l:numLastLine = (line("$") - 1)
      "
      while l:numCurrLine >= 0 && l:numCurrLine <= l:numLastLine
        if getline(l:numCurrLine) =~ '\c\v^\s*;\s*((end)?fold)@!'
          let l:numNextNoneCommentLine = search('\v\c^\s*([^ \t;]|;\s*(end)?fold)',"nW")
          if l:numNextNoneCommentLine == 0
            execute "normal" "gqG"
          elseif l:numNextNoneCommentLine-l:numCurrLine <= 1
            execute "normal" "gqq"
          else
            execute "normal" (l:numNextNoneCommentLine-l:numCurrLine-1)."gqj"
          endif
        endif
        " check next line
        let l:searchnextcomment = search('\c\v^\s*;\s*((end)?fold)@!',"W")
        if l:searchnextcomment == 0
          execute "normal" "G"
        endif
        let l:numCurrLine = line(".")
        let l:numLastLine = (line("$") - 1)
      endwhile
      "
    endfunction " KrlFormatComments

  endif
endif " format comments

" global function
" gd mimic; NOTE: may not work well without g:krlNoKeyWord=1 and/or g:krlNoPath=1 as well as with syntax off
if !exists("*KrlGoDefinition()")

  function KrlGoDefinition()
    "
    execute "lclose"
    call setloclist(0,[])
    execute "cclose"
    call setqflist([])
    let l:declPrefix = '\c\v^\s*((decl\s+)?(global\s+)?(const\s+)?(bool|int|real|char|frame|pos|axis|e6pos|e6axis|signal|struc|enum)\s+|decl\s+(global\s+)?(const\s+)?\w+\s+)'
    "
    " suche das naechste wort
    if search('\w','cW',line("."))
      "
      let l:currentWord = s:KrlCurrentWordIs()
      "
      if l:currentWord =~ '^sysvar.*'
        let l:currentWord = substitute(l:currentWord,'^sysvar','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a KSS VARIABLE, STRUCT or ENUM"])
        return s:KrlSearchSysvar(l:declPrefix,l:currentWord)
        "
      elseif l:currentWord =~ '^var.*'
        let l:currentWord = substitute(l:currentWord,'^var','','')
        let l:currentWord = substitute(l:currentWord,'\(\w\+\)\$','\1\\$','g') " escape embeddend dollars in var name (e.g. TMP_$STOPM) 
        call s:KrlVerboseEcho([l:currentWord,"appear to be a user defined VARIABLE, STRUCT or ENUM"])
        return s:KrlSearchVar(l:declPrefix,l:currentWord)
        "
      elseif l:currentWord =~ '\v^(sys)?(proc|func)'
        let l:type = "DEF"
        if l:currentWord =~ '^sys'
          let l:type = "KSS " . l:type
        endif
        if l:currentWord =~ '^\v(sys)?func'
          let l:type = l:type . "FCT" 
        endif
        let l:currentWord = substitute(l:currentWord,'\v^(sys)?(proc|func)','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a ".l:type])
        return s:KrlSearchProc(l:currentWord)
        "
      elseif l:currentWord =~ '^enumval.*'
        let l:currentWord = substitute(l:currentWord,'^enumval','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be an ENUM VALUE. No search performed."])
      elseif l:currentWord =~ '^num.*'
        let l:currentWord = substitute(l:currentWord,'^num','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a NUMBER. No search performed."])
      elseif l:currentWord =~ '^string.*'
        let l:currentWord = substitute(l:currentWord,'^string','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a STRING. No search performed."])
      elseif l:currentWord =~ '^comment.*'
        let l:currentWord = substitute(l:currentWord,'^comment','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a COMMENT. No search performed."])
      elseif l:currentWord =~ '^inst.*'
        let l:currentWord = substitute(l:currentWord,'^inst','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a KRL KEYWORD. No search performed."])
      elseif l:currentWord =~ '^bool.*'
        let l:currentWord = substitute(l:currentWord,'^bool','','')
        call s:KrlVerboseEcho([l:currentWord,"appear to be a BOOL VALUE. No search performed."])
      else
        let l:currentWord = substitute(l:currentWord,'^none','','')
        call s:KrlVerboseEcho([l:currentWord,"Could not determine typ of current word. No search performed."])
      endif
      return -1
      "
    else " kein wort gefunden
      call s:KrlVerboseEcho("Nothing found at or after current cursor pos, which could have a declaration. No search performed.")
      return -1
      "
    endif
    "
  endfunction " KrlGoDefinition

  function s:KrlCurrentWordIs()
    " returns the string "<type><name>" depending on the word under the curser
    "
    let l:numLine = line(".")
    let l:strLine = getline(".")
    "
    " position the cursor at the start of the current word
    if search('\<','bcsW',l:numLine)
      "
      " init
      let l:numCol = col(".")
      let l:currentChar = strpart(l:strLine, l:numCol-1, 1)
      let l:strUntilCursor = strpart(l:strLine, 0, l:numCol-1)
      let l:lenStrUntilCursor = strlen(l:strUntilCursor)
      "
      " find next char
      if search('\>\s*.',"eW",l:numLine)
        let l:nextChar = strpart(l:strLine, col(".")-1, 1)
        let l:colNextChar = col(".")
      else
        execute "normal" "$"
        let l:nextChar = ""
        let l:colNextChar = col(".")+1
      endif
      "
      " set cursor back to start of word
      call cursor(l:numLine,l:numCol)
      "
      " get word at cursor
      let l:word = expand("<cword>")
      "
      " count string chars " before the current char
      let l:i = 0
      let l:countStrChr = 0
      while l:i < l:lenStrUntilCursor
        let l:i = stridx(l:strUntilCursor, "\"", l:i)
        if l:i >= 0
          let l:i = l:i+1
          let l:countStrChr = l:countStrChr+1
        else
          let l:i = l:lenStrUntilCursor+1
        endif
      endwhile
      let l:countStrChr = l:countStrChr%2
      "
      " return something
      if search(';','bcnW',l:numLine)
        return ("comment" . l:word)
        "
      elseif l:countStrChr == 1
        return ("string" . l:word)
        "
      elseif l:currentChar == "$"
        return ("sysvar" . l:word)
        "
      elseif l:currentChar == "#"
        return ("enumval" . l:word)
        "
      elseif l:word =~ '\v\c^(true|false)>'
        return ("bool" . l:word)
        "
      elseif l:currentChar =~ '\d' || 
            \(l:word=~'^[bB][10]\+$' && 
              \   (synIDattr(synID(line("."),col("."),0),"name")=="krlBinaryInt" 
              \ || synIDattr(synID(line("."),col("."),0),"name")=="") 
            \|| l:word=~'^[hH][0-9a-fA-F]\+$' && 
              \   (synIDattr(synID(line("."),col("."),0),"name")=="krlHexInt" 
              \ || synIDattr(synID(line("."),col("."),0),"name")=="")
            \) && l:nextChar == "'"
        return ("num" . l:word)
        "
      elseif l:nextChar == "(" && 
            \(  synIDattr(synID(line("."),col("."),0),"name")=="krlFunction"
            \|| synIDattr(synID(line("."),col("."),0),"name")=="krlBuildInFunction"
            \|| synIDattr(synID(line("."),col("."),0),"name")==""
            \)
        if search('\c\v(<do>|<def>)\s*'.l:word,'bnW',l:numLine) || !search('[^\t ]','bnW',l:numLine)
          if synIDattr(synID(line("."),col("."),0),"name") != "krlBuildInFunction"
            return ("proc" . l:word)
            "
          else
            return ("sysproc" . l:word)
            "
          endif
        else
          if synIDattr(synID(line("."),col("."),0),"name") != "krlBuildInFunction"
            return ("func" . l:word)
            "
          else
            return ("sysfunc" . l:word)
            "
          endif
        endif
      else
        if        synIDattr(synID(line("."),col("."),0),"name") != "krlNames"
              \&& synIDattr(synID(line("."),col("."),0),"name") != "krlSysvars"
              \&& synIDattr(synID(line("."),col("."),0),"name") != "krlStructure"
              \&& synIDattr(synID(line("."),col("."),0),"name") != "krlEnum"
              \&& synIDattr(synID(line("."),col("."),0),"name") != ""
          return ("inst" . l:word)
          "
        else
          return ("var" . l:word)
          "
        endif
      endif
      " end return something
      "
    else
      return "none"
      "
    endif
  endfunction " s:KrlCurrentWordIs

  function s:KrlSearchPathForPattern(strPattern,path)
    let l:cmd = ':noautocmd 1vimgrep /' . a:strPattern . '/j ' . a:path
    try
      execute l:cmd
    catch /^Vim\%((\a\+)\)\=:E303/
      call s:KrlVerboseEcho(":vimgrep stopped with E303. No match found")
      return -1
    catch /^Vim\%((\a\+)\)\=:E480/
      call s:KrlVerboseEcho(":vimgrep stopped with E480. No match found")
      return -1
    catch /^Vim\%((\a\+)\)\=:E683/
      call s:KrlVerboseEcho(":vimgrep stopped with E683. No match found")
      return -1
    endtry
    if getqflist() != []
      execute ":cwindow 4"
      execute "set ft=krl"
      return 0
    else
      call s:KrlVerboseEcho("No match found")
      return -1
    endif
  endfunction " s:KrlSearchPathForPattern

  function s:KrlPathWithGlobalDataLists()
    let l:cmd = ':noautocmd lvimgrep /\c\v^\s*defdat\s+(\w+\s+public|\$\w+)/j ' . s:KrlPreparePath(&path,'*.dat')
    try
      execute l:cmd
    catch /^Vim\%((\a\+)\)\=:E480/
      call s:KrlVerboseEcho(":lvimgrep stopped with E480! No global data lists found in \'path\' Will search only local .dat file.")
      return ' '
    catch /^Vim\%((\a\+)\)\=:E683/
      call s:KrlVerboseEcho(":lvimgrep stops with E683! No global data lists found in \'path\'. Will search only local .dat file.")
      return ' '
    endtry
    let l:locationList = getloclist(0)
    let l:path = ' '
    for l:loc in l:locationList
      let l:path = l:path . fnameescape(bufname(l:loc.bufnr)) . " "
    endfor
    return l:path
  endfunction " s:KrlPathWithGlobalDataLists

  function s:KrlPreparePath(path,ending)
    let l:path = substitute(a:path,'$',' ','') " make sure that space is the last char
    let l:path = substitute(l:path,',',' ','g') " literal commas in a:path do not work
    let l:path = substitute(l:path, '\*\* ', '**/'.a:ending.' ', 'g')
    let l:path = substitute(l:path, '\.\. ', '../'.a:ending.' ', 'g')
    let l:path = substitute(l:path, '\. ',    './'.a:ending.' ', 'g')
    let l:path = substitute(l:path, '[\\/] ',  '/'.a:ending.' ', 'g')
    return l:path
  endfunction " s:KrlPreparePath

  function s:KrlSearchSysvar(declPrefix,currentWord)
    "
    " a:currentWord starts with '$' so we need '\' at the end of the pattern
    call s:KrlVerboseEcho("Search global data lists...")
    if (s:KrlSearchPathForPattern(a:declPrefix.'\'.a:currentWord.">",s:KrlPathWithGlobalDataLists()) == 0)
      call s:KrlVerboseEcho("Found global data list declaration. The quickfix window will open. See :he quickfix-window")
      return 0
    endif
    return -1
  endfunction "s:KrlSearchSysvar 

  function s:KrlSearchVar(declPrefix,currentWord)
    "
    " first search for local declartion
    call s:KrlVerboseEcho("Search def(fct)? local declaration...")
    let l:numLine = line(".")
    let l:numCol = col(".")
    let l:numDefLine = search('\v\c^\s*(global\s+)?<def(fct|dat)?>','bW')
    let l:numEndLine = search('\v\c^\s*<end(fct|dat)?>','nW')
    if search(a:declPrefix.'(\w|,|\s|\[|\])*<'.a:currentWord.">","eW",l:numEndLine)
      call s:KrlVerboseEcho("Found def(fct|dat)? local declaration. Get back where you came from with ''")
      execute "normal" "b"
      return 0
      "
    else
      call s:KrlVerboseEcho("No match found")
      call cursor(l:numLine,l:numCol)
    endif
    " 
    " search corrosponding dat file
    call s:KrlVerboseEcho("Search local data list...")
    let l:filename = substitute(fnameescape(bufname("%")),'\c\.src$','.dat','')
    if filereadable(l:filename) " suppress message about missing file
      if (s:KrlSearchPathForPattern(a:declPrefix.'<'.a:currentWord.">",l:filename) == 0)
        call s:KrlVerboseEcho("Found local data list declaration. The quickfix window will open. See :he quickfix-window")
        return 0
        "
      endif
    else
      call s:KrlVerboseEcho(["File ",l:filename," not readable"])
    endif " search corrosponding dat file
    " 
    " third search global data lists
    call s:KrlVerboseEcho("Search global data lists...")
    if (s:KrlSearchPathForPattern(a:declPrefix.'<'.a:currentWord.">",s:KrlPathWithGlobalDataLists()) == 0)
      call s:KrlVerboseEcho("Found global data list declaration. The quickfix window will open. See :he quickfix-window")
      return 0
      "
    endif
    "
    return -1
  endfunction " s:KrlSearchVar

  function s:KrlSearchProc(currentWord)
    "
    " first search for local def(fct)? declartion
    call s:KrlVerboseEcho("Search def(fct)? definitions in current file...")
    let l:numLine = line(".")
    let l:numCol = col(".")
    execute 0
    if search('\c\v^\s*(global\s+)?def(fct\s+\w+)?\s+'.a:currentWord.'>','cew',"$")
      call s:KrlVerboseEcho("Found def(fct)? local declaration. Get back where you came from with ''")
      execute "normal" "b"
      return 0
      "
    else
      call s:KrlVerboseEcho("No match found")
      call cursor(l:numLine,l:numCol)
    endif
    "
    " second search src file name = a:currentWord
    call s:KrlVerboseEcho("Search .src files in &path...")
    let l:path = s:KrlPreparePath(&path,a:currentWord.'.src')
    if !filereadable('./'.a:currentWord.'.src') " suppress message about missing file
      let l:path = substitute(l:path, '\.[\\/]'.a:currentWord.'.src ', ' ','g')
    endif
    if (s:KrlSearchPathForPattern('\c\v^\s*(global\s+)?def(fct\s+\w+)?\s+'.a:currentWord.">",l:path) == 0)
      call s:KrlVerboseEcho("Found src file. The quickfix window will open. See :he quickfix-window")
      return 0
      "
    endif
    "
    " third search global def(fct)?
    call s:KrlVerboseEcho("Search global def(fct)? definitions in &path...")
    if (s:KrlSearchPathForPattern('\c\v^\s*global\s+def(fct\s+\w+)?\s+'.a:currentWord.">",s:KrlPreparePath(&path,'*.src')) == 0)
      call s:KrlVerboseEcho("Found global def(fct)? declaration. The quickfix window will open. See :he quickfix-window")
      return 0
      "
    endif
    "
    return -1
  endfunction " s:KrlSearchProc

" gd mimic
endif " !exists("*KrlGoDefinition()")

let &cpo = s:keepcpo
unlet s:keepcpo

" vim:sw=2 sts=2 et
